package org.hawk.gwt.ppc.reflect.emul.java.lang;

import java.lang.annotation.Annotation;

import org.hawk.gwt.ppc.reflect.lang.ClassDelegate;

/**
 * Represents information about package for the {@link ClassDelegate}.
 * Overrides initial {@link Package} methods.
 * @author alex.bereznevatiy@gmail.com
 *
 */
public final class Package {
	private String name;
	
	private  Package(String name) {
		this.name = name;
	}


	/**
     * Return the name of this package.
     *
     * @return  The fully-qualified name of this package as defined in the
     *          <em>Java Language Specification, Third Edition</em>
     *          <a href="http://java.sun.com/docs/books/jls/third_edition/html/names.html#6.5.3">
     *          &sect;6.5.3</a>, for example, <tt>java.lang</tt>
     */
    public String getName() {
    	return name;
    }


    /**
     * Return the title of the specification that this package implements.
     * @return the specification title, null is returned if it is not known.
     */
    public String getSpecificationTitle() {
    	return null;
    }

    /**
     * Returns the version number of the specification
     * that this package implements.
     * This version string must be a sequence of positive decimal
     * integers separated by "."'s and may have leading zeros.
     * When version strings are compared the most significant
     * numbers are compared.
     * @return the specification version, null is returned if it is not known.
     */
    public String getSpecificationVersion() {
    	return null;
    }

    /**
     * Return the name of the organization, vendor,
     * or company that owns and maintains the specification
     * of the classes that implement this package.
     * @return the specification vendor, null is returned if it is not known.
     */
    public String getSpecificationVendor() {
    	return null;
    }

    /**
     * Return the title of this package.
     * @return the title of the implementation, null is returned if it is not known.
     */
    public String getImplementationTitle() {
    	return null;
    }

    /**
     * Return the version of this implementation. It consists of any string
     * assigned by the vendor of this implementation and does
     * not have any particular syntax specified or expected by the Java
     * runtime. It may be compared for equality with other
     * package version strings used for this implementation
     * by this vendor for this package.
     * @return the version of the implementation, null is returned if it is not known.
     */
    public String getImplementationVersion() {
    	return null;
    }

    /**
     * Returns the name of the organization,
     * vendor or company that provided this implementation.
     * @return the vendor that implemented this package..
     */
    public String getImplementationVendor() {
    	return null;
    }

    /**
     * Returns true if this package is sealed.
     *
     * @return true if the package is sealed, false otherwise
     */
    public boolean isSealed() {
    	throw new UnsupportedOperationException();    
	}

    /**
     * Returns true if this package is sealed with respect to the specified
     * code source url.
     *
     * @param url the code source url
     * @return true if this package is sealed with respect to url
     */
    //NOT SUPPORTED AT ALL
    /*public boolean isSealed(URL url) {
    	throw new UnsupportedOperationException();
    }*/

    /**
     * Compare this package's specification version with a
     * desired version. It returns true if
     * this packages specification version number is greater than or equal
     * to the desired version number. <p>
     *
     * Version numbers are compared by sequentially comparing corresponding
     * components of the desired and specification strings.
     * Each component is converted as a decimal integer and the values
     * compared.
     * If the specification value is greater than the desired
     * value true is returned. If the value is less false is returned.
     * If the values are equal the period is skipped and the next pair of
     * components is compared.
     *
     * @param desired the version string of the desired version.
     * @return true if this package's version number is greater
     * 		than or equal to the desired version number
     *
     * @exception NumberFormatException if the desired or current version
     *		is not of the correct dotted form.
     */
    public boolean isCompatibleWith(String desired)
	throws NumberFormatException
    {
    	throw new UnsupportedOperationException();
    }

    /**
     * Find a package by name in the callers <code>ClassLoader</code> instance.
     * The callers <code>ClassLoader</code> instance is used to find the package
     * instance corresponding to the named class. If the callers
     * <code>ClassLoader</code> instance is null then the set of packages loaded
     * by the system <code>ClassLoader</code> instance is searched to find the
     * named package. <p>
     *
     * Packages have attributes for versions and specifications only if the class
     * loader created the package instance with the appropriate attributes. Typically,
     * those attributes are defined in the manifests that accompany the classes.
     *
     * @param name a package name, for example, java.lang.
     * @return the package of the requested name. It may be null if no package
     * 		information is available from the archive or codebase.
     */
    public static Package getPackage(String name) {
    	return new Package(name);
    }

    /**
     * Get all the packages currently known for the caller's <code>ClassLoader</code>
     * instance.  Those packages correspond to classes loaded via or accessible by
     * name to that <code>ClassLoader</code> instance.  If the caller's
     * <code>ClassLoader</code> instance is the bootstrap <code>ClassLoader</code>
     * instance, which may be represented by <code>null</code> in some implementations,
     * only packages corresponding to classes loaded by the bootstrap
     * <code>ClassLoader</code> instance will be returned.
     *
     * @return a new array of packages known to the callers <code>ClassLoader</code>
     * instance.  An zero length array is returned if none are known.
     */
    public static Package[] getPackages() {
    	throw new UnsupportedOperationException();
    }

    /**
     * Return the hash code computed from the package name.
     * @return the hash code computed from the package name.
     */
    public int hashCode(){
    	return name.hashCode();
    }

    /**
     * Returns the string representation of this Package.
     * Its value is the string "package " and the package name.
     * If the package title is defined it is appended.
     * If the package version is defined it is appended.
     * @return the string representation of the package.
     */
    public String toString() {
        return "package "+name;
    }

    /**
     * @throws NullPointerException {@inheritDoc}
     * @since 1.5
     */
    public <A extends Annotation> A getAnnotation(Class<A> annotationClass) {
    	throw new UnsupportedOperationException();
    }

    /**
     * @throws NullPointerException {@inheritDoc}
     * @since 1.5
     */
    public boolean isAnnotationPresent(
        Class<? extends Annotation> annotationClass) {
    	throw new UnsupportedOperationException();
    }

    /**
     * @since 1.5
     */
    public Annotation[] getAnnotations() { 
    	throw new UnsupportedOperationException();
    }

    /**
     * @since 1.5
     */
    public Annotation[] getDeclaredAnnotations()  {
    	throw new UnsupportedOperationException();
    }
}
